@@ -1,5 +1,5 @@
 /* This file is part of the 'stringi' package for R.
- * Copyright (C) 2013-2015, Marek Gagolewski and Bartek Tartanus
+ * Copyright (c) 2013-2018, Marek Gagolewski and other contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -206,6 +206,9 @@ SEXP stri_prepare_arg_list_string(SEXP x, const char* argname)
  *
  * @version 0.5-1 (Marek Gagolewski, 2015-05-01)
  *        #154 - the class attribute set fires up an as.xxxx call
+ *
+ * @version 1.2.1 (Marek Gagolewski, 2018-04-21)
+ *    #285: warn if coercing from a non-trivial list
  */
 SEXP stri_prepare_arg_string(SEXP x, const char* argname)
 {
@@ -216,15 +219,25 @@ SEXP stri_prepare_arg_string(SEXP x, const char* argname)
    {
       SEXP call;
       PROTECT(call = Rf_lang2(Rf_install("as.character"), x));
-      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark it's encoding manually
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
       UNPROTECT(2);
       return x;
    }
    else if (Rf_isVectorList(x) || isObject(x))
    {
+      if (Rf_isVectorList(x)) {
+         R_len_t nv = LENGTH(x);
+         for (R_len_t i=0; i<nv; ++i) {
+            SEXP cur = VECTOR_ELT(x, i);
+            if (!(Rf_isVectorAtomic(cur) && LENGTH(cur) == 1)) {
+               Rf_warning(MSG__WARN_LIST_COERCION);
+               break;
+            }
+         }
+      }
       SEXP call;
       PROTECT(call = Rf_lang2(Rf_install("as.character"), x));
-      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark it's encoding manually
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
       UNPROTECT(2);
       return x;
    }
@@ -266,6 +279,9 @@ SEXP stri_prepare_arg_string(SEXP x, const char* argname)
  *
  * @version 0.5-1 (Marek Gagolewski, 2015-05-01)
  *        #154 - the class attribute set fires up an as.xxxx call
+ *
+ * @version 1.2.1 (Marek Gagolewski, 2018-04-21)
+ *    #285: warn if coercing from a non-trivial list
  */
 SEXP stri_prepare_arg_double(SEXP x, const char* argname)
 {
@@ -276,16 +292,26 @@ SEXP stri_prepare_arg_double(SEXP x, const char* argname)
    {
       SEXP call;
       PROTECT(call = Rf_lang2(Rf_install("as.character"), x));
-      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark it's encoding manually
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
       PROTECT(x = Rf_coerceVector(x, REALSXP));
       UNPROTECT(3);
       return x;
    }
    else if (Rf_isVectorList(x) || isObject(x))
    {
+      if (Rf_isVectorList(x)) {
+         R_len_t nv = LENGTH(x);
+         for (R_len_t i=0; i<nv; ++i) {
+            SEXP cur = VECTOR_ELT(x, i);
+            if (!(Rf_isVectorAtomic(cur) && LENGTH(cur) == 1)) {
+               Rf_warning(MSG__WARN_LIST_COERCION);
+               break;
+            }
+         }
+      }
       SEXP call;
       PROTECT(call = Rf_lang2(Rf_install("as.double"), x));
-      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark it's encoding manually
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
       UNPROTECT(2);
       return x;
    }
@@ -367,6 +393,9 @@ SEXP stri_prepare_arg_POSIXct(SEXP x, const char* argname)
  *
  * @version 0.5-1 (Marek Gagolewski, 2015-05-01)
  *        #154 - the class attribute set fires up an as.xxxx call
+ *
+ * @version 1.2.1 (Marek Gagolewski, 2018-04-21)
+ *    #285: warn if coercing from a non-trivial list
  */
 SEXP stri_prepare_arg_integer(SEXP x, const char* argname)
 {
@@ -377,16 +406,26 @@ SEXP stri_prepare_arg_integer(SEXP x, const char* argname)
    {
       SEXP call;
       PROTECT(call = Rf_lang2(Rf_install("as.character"), x));
-      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark it's encoding manually
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
       PROTECT(x = Rf_coerceVector(x, INTSXP));
       UNPROTECT(3);
       return x;
    }
    else if (Rf_isVectorList(x) || isObject(x))
    {
+      if (Rf_isVectorList(x)) {
+         R_len_t nv = LENGTH(x);
+         for (R_len_t i=0; i<nv; ++i) {
+            SEXP cur = VECTOR_ELT(x, i);
+            if (!(Rf_isVectorAtomic(cur) && LENGTH(cur) == 1)) {
+               Rf_warning(MSG__WARN_LIST_COERCION);
+               break;
+            }
+         }
+      }
       SEXP call;
       PROTECT(call = Rf_lang2(Rf_install("as.integer"), x));
-      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark it's encoding manually
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
       UNPROTECT(2);
       return x;
    }
@@ -426,6 +465,9 @@ SEXP stri_prepare_arg_integer(SEXP x, const char* argname)
  *
  * @version 0.5-1 (Marek Gagolewski, 2015-05-01)
  *        #154 - the class attribute set fires up an as.xxxx call
+ *
+ * @version 1.2.1 (Marek Gagolewski, 2018-04-21)
+ *    #285: warn if coercing from a non-trivial list
  */
 SEXP stri_prepare_arg_logical(SEXP x, const char* argname)
 {
@@ -436,16 +478,26 @@ SEXP stri_prepare_arg_logical(SEXP x, const char* argname)
    {
       SEXP call;
       PROTECT(call = Rf_lang2(Rf_install("as.character"), x));
-      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark it's encoding manually
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
       PROTECT(x = Rf_coerceVector(x, LGLSXP));
       UNPROTECT(3);
       return x;
    }
    else if (Rf_isVectorList(x) || isObject(x))
    {
+      if (Rf_isVectorList(x)) {
+         R_len_t nv = LENGTH(x);
+         for (R_len_t i=0; i<nv; ++i) {
+            SEXP cur = VECTOR_ELT(x, i);
+            if (!(Rf_isVectorAtomic(cur) && LENGTH(cur) == 1)) {
+               Rf_warning(MSG__WARN_LIST_COERCION);
+               break;
+            }
+         }
+      }
       SEXP call;
       PROTECT(call = Rf_lang2(Rf_install("as.logical"), x));
-      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark it's encoding manually
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
       UNPROTECT(2);
       return x;
    }
@@ -482,6 +534,9 @@ SEXP stri_prepare_arg_logical(SEXP x, const char* argname)
  *
  * @version 0.5-1 (Marek Gagolewski, 2015-05-01)
  *        #154 - the class attribute set fires up an as.xxxx call
+ *
+ * @version 1.2.1 (Marek Gagolewski, 2018-04-21)
+ *    #285: warn if coercing from a non-trivial list
  */
 SEXP stri_prepare_arg_raw(SEXP x, const char* argname)
 {
@@ -492,16 +547,26 @@ SEXP stri_prepare_arg_raw(SEXP x, const char* argname)
    {
       SEXP call;
       PROTECT(call = Rf_lang2(Rf_install("as.character"), x));
-      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark it's encoding manually
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
       PROTECT(x = Rf_coerceVector(x, RAWSXP));
       UNPROTECT(3);
       return x;
    }
    else if (Rf_isVectorList(x) || isObject(x))
    {
+      if (Rf_isVectorList(x)) {
+         R_len_t nv = LENGTH(x);
+         for (R_len_t i=0; i<nv; ++i) {
+            SEXP cur = VECTOR_ELT(x, i);
+            if (!(Rf_isVectorAtomic(cur) && LENGTH(cur) == 1)) {
+               Rf_warning(MSG__WARN_LIST_COERCION);
+               break;
+            }
+         }
+      }
       SEXP call;
       PROTECT(call = Rf_lang2(Rf_install("as.raw"), x));
-      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark it's encoding manually
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
       UNPROTECT(2);
       return x;
    }
@@ -532,17 +597,60 @@ SEXP stri_prepare_arg_raw(SEXP x, const char* argname)
  *
  * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
  *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
+ *
+ * @version 1.2.1 (Marek Gagolewski, 2018-04-21)
+ *    #285: warn if coercing from a non-trivial list
  */
 SEXP stri_prepare_arg_string_1(SEXP x, const char* argname)
 {
    if ((SEXP*)argname == (SEXP*)R_NilValue)
       argname = "<noname>";
 
-   PROTECT(x = stri_prepare_arg_string(x, argname));
+   int nprotect = 0;
+   if (Rf_isFactor(x))
+   {
+      SEXP call;
+      PROTECT(call = Rf_lang2(Rf_install("as.character"), x));
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
+      nprotect = 2;
+   }
+   else if (Rf_isVectorList(x) || isObject(x))
+   {
+      if (Rf_isVectorList(x)) {
+         R_len_t nv = LENGTH(x);
+         for (R_len_t i=0; i<nv; ++i) {
+            SEXP cur = VECTOR_ELT(x, i);
+            if (!(Rf_isVectorAtomic(cur) && LENGTH(cur) == 1)) {
+               Rf_warning(MSG__WARN_LIST_COERCION);
+               break;
+            }
+         }
+      }
+      SEXP call;
+      PROTECT(call = Rf_lang2(Rf_install("as.character"), x));
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
+      nprotect = 2;
+   }
+   else if ((bool)isString(x))
+      nprotect = 0;
+   else if (Rf_isVectorAtomic(x) || isNull(x)) {
+      PROTECT(x = Rf_coerceVector(x, STRSXP));
+      nprotect = 1;
+   }
+   else if ((bool)isSymbol(x)) {
+      PROTECT(x = Rf_ScalarString(PRINTNAME(x)));
+      nprotect = 1;
+   }
+   else {
+      Rf_error(MSG__ARG_EXPECTED_STRING, argname); // allowed here
+      return x; // avoid compiler warning
+   }
+
+
    R_len_t nx = LENGTH(x);
 
    if (nx <= 0) {
-      UNPROTECT(1);
+      UNPROTECT(nprotect);
       Rf_error(MSG__ARG_EXPECTED_NOT_EMPTY, argname); // allowed here
       // won't come here anyway
       return x; // avoid compiler warning
@@ -551,12 +659,13 @@ SEXP stri_prepare_arg_string_1(SEXP x, const char* argname)
       Rf_warning(MSG__ARG_EXPECTED_1_STRING, argname);
       SEXP xold = x;
       PROTECT(x = Rf_allocVector(STRSXP, 1));
+      nprotect++;
       SET_STRING_ELT(x, 0, STRING_ELT(xold, 0));
-      UNPROTECT(2);
+      UNPROTECT(nprotect);
       return x;
    }
    else { // if (nx == 1)
-      UNPROTECT(1);
+      UNPROTECT(nprotect);
       return x;
    }
 }
@@ -579,17 +688,56 @@ SEXP stri_prepare_arg_string_1(SEXP x, const char* argname)
  *
  * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
  *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
+ *
+ * @version 1.2.1 (Marek Gagolewski, 2018-04-21)
+ *    #285: warn if coercing from a non-trivial list
  */
 SEXP stri_prepare_arg_double_1(SEXP x, const char* argname)
 {
    if ((SEXP*)argname == (SEXP*)R_NilValue)
       argname = "<noname>";
 
-   PROTECT(x = stri_prepare_arg_double(x, argname));
+   int nprotect = 0;
+   if (Rf_isFactor(x))
+   {
+      SEXP call;
+      PROTECT(call = Rf_lang2(Rf_install("as.character"), x));
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
+      PROTECT(x = Rf_coerceVector(x, REALSXP));
+      nprotect = 3;
+   }
+   else if (Rf_isVectorList(x) || isObject(x))
+   {
+      if (Rf_isVectorList(x)) {
+         R_len_t nv = LENGTH(x);
+         for (R_len_t i=0; i<nv; ++i) {
+            SEXP cur = VECTOR_ELT(x, i);
+            if (!(Rf_isVectorAtomic(cur) && LENGTH(cur) == 1)) {
+               Rf_warning(MSG__WARN_LIST_COERCION);
+               break;
+            }
+         }
+      }
+      SEXP call;
+      PROTECT(call = Rf_lang2(Rf_install("as.double"), x));
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
+      nprotect = 2;
+   }
+   else if ((bool)isReal(x))
+      nprotect = 0;
+   else if (Rf_isVectorAtomic(x) || isNull(x)) {
+      PROTECT(x = Rf_coerceVector(x, REALSXP));
+      nprotect = 1;
+   }
+   else {
+      Rf_error(MSG__ARG_EXPECTED_NUMERIC, argname); // allowed here
+      return x; // avoid compiler warning
+   }
+
    R_len_t nx = LENGTH(x);
 
    if (nx <= 0) {
-      UNPROTECT(1);
+      UNPROTECT(nprotect);
       Rf_error(MSG__ARG_EXPECTED_NOT_EMPTY, argname); // allowed here
       // won't come here anyway
       return x; // avoid compiler warning
@@ -598,12 +746,13 @@ SEXP stri_prepare_arg_double_1(SEXP x, const char* argname)
       Rf_warning(MSG__ARG_EXPECTED_1_NUMERIC, argname);
       double x0 = REAL(x)[0];
       PROTECT(x = Rf_allocVector(REALSXP, 1));
+      nprotect++;
       REAL(x)[0] = x0;
-      UNPROTECT(2);
+      UNPROTECT(nprotect);
       return x;
    }
    else {// if (nx == 1)
-      UNPROTECT(1);
+      UNPROTECT(nprotect);
       return x;
    }
 }
@@ -626,17 +775,56 @@ SEXP stri_prepare_arg_double_1(SEXP x, const char* argname)
  *
  * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
  *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
+ *
+ * @version 1.2.1 (Marek Gagolewski, 2018-04-21)
+ *    #285: warn if coercing from a non-trivial list
  */
 SEXP stri_prepare_arg_integer_1(SEXP x, const char* argname)
 {
    if ((SEXP*)argname == (SEXP*)R_NilValue)
       argname = "<noname>";
 
-   PROTECT(x = stri_prepare_arg_integer(x, argname));
+   int nprotect = 0;
+   if (Rf_isFactor(x)) // factors must be checked first (as they are currently represented as integer vectors)
+   {
+      SEXP call;
+      PROTECT(call = Rf_lang2(Rf_install("as.character"), x));
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
+      PROTECT(x = Rf_coerceVector(x, INTSXP));
+      nprotect = 3;
+   }
+   else if (Rf_isVectorList(x) || isObject(x))
+   {
+      if (Rf_isVectorList(x)) {
+         R_len_t nv = LENGTH(x);
+         for (R_len_t i=0; i<nv; ++i) {
+            SEXP cur = VECTOR_ELT(x, i);
+            if (!(Rf_isVectorAtomic(cur) && LENGTH(cur) == 1)) {
+               Rf_warning(MSG__WARN_LIST_COERCION);
+               break;
+            }
+         }
+      }
+      SEXP call;
+      PROTECT(call = Rf_lang2(Rf_install("as.integer"), x));
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
+      nprotect = 2;
+   }
+   else if (Rf_isInteger(x))
+      nprotect = 0;
+   else if (Rf_isVectorAtomic(x) || isNull(x)) {
+      PROTECT(x = Rf_coerceVector(x, INTSXP));
+      nprotect = 1;
+   }
+   else {
+      Rf_error(MSG__ARG_EXPECTED_INTEGER, argname); //allowed here
+      return x; // avoid compiler warning
+   }
+
    R_len_t nx = LENGTH(x);
 
    if (nx <= 0) {
-      UNPROTECT(1);
+      UNPROTECT(nprotect);
       Rf_error(MSG__ARG_EXPECTED_NOT_EMPTY, argname); // allowed here
       // won't come here anyway
       return x; // avoid compiler warning
@@ -645,12 +833,13 @@ SEXP stri_prepare_arg_integer_1(SEXP x, const char* argname)
       Rf_warning(MSG__ARG_EXPECTED_1_INTEGER, argname);
       int x0 = INTEGER(x)[0];
       PROTECT(x = Rf_allocVector(INTSXP, 1));
+      nprotect++;
       INTEGER(x)[0] = x0;
-      UNPROTECT(2);
+      UNPROTECT(nprotect);
       return x;
    }
    else { // if (nx == 1)
-      UNPROTECT(1);
+      UNPROTECT(nprotect);
       return x;
    }
 }
@@ -673,17 +862,59 @@ SEXP stri_prepare_arg_integer_1(SEXP x, const char* argname)
  *
  * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
  *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
+ *
+ * @version 1.2.1 (Marek Gagolewski, 2018-04-21)
+ *    #285: warn if coercing from a non-trivial list
  */
 SEXP stri_prepare_arg_logical_1(SEXP x, const char* argname)
 {
+   int nprotect = 0;
+
    if ((SEXP*)argname == (SEXP*)R_NilValue)
       argname = "<noname>";
 
-   PROTECT(x = stri_prepare_arg_logical(x, argname));
+   if (Rf_isFactor(x))
+   {
+      SEXP call;
+      PROTECT(call = Rf_lang2(Rf_install("as.character"), x));
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
+      PROTECT(x = Rf_coerceVector(x, LGLSXP));
+      nprotect = 3;
+   }
+   else if (Rf_isVectorList(x) || isObject(x))
+   {
+      if (Rf_isVectorList(x)) {
+         R_len_t nv = LENGTH(x);
+         for (R_len_t i=0; i<nv; ++i) {
+            SEXP cur = VECTOR_ELT(x, i);
+            if (!(Rf_isVectorAtomic(cur) && LENGTH(cur) == 1)) {
+               Rf_warning(MSG__WARN_LIST_COERCION);
+               break;
+            }
+         }
+      }
+      SEXP call;
+      PROTECT(call = Rf_lang2(Rf_install("as.logical"), x));
+      PROTECT(x = Rf_eval(call, R_GlobalEnv)); // this will mark its encoding manually
+      nprotect = 2;
+   }
+   else if ((bool)isLogical(x)) {
+      nprotect = 0;
+      // do nothing
+   }
+   else if (Rf_isVectorAtomic(x) || isNull(x)) {
+      PROTECT(x = Rf_coerceVector(x, LGLSXP));
+      nprotect = 1;
+   }
+   else {
+      Rf_error(MSG__ARG_EXPECTED_LOGICAL, argname); // allowed here
+      return x; // avoid compiler warning
+   }
+
    R_len_t nx = LENGTH(x);
 
    if (nx <= 0) {
-      UNPROTECT(1);
+      UNPROTECT(nprotect);
       Rf_error(MSG__ARG_EXPECTED_NOT_EMPTY, argname); // allowed here
       // won't come here anyway
       return x; // avoid compiler warning
@@ -692,12 +923,13 @@ SEXP stri_prepare_arg_logical_1(SEXP x, const char* argname)
       Rf_warning(MSG__ARG_EXPECTED_1_LOGICAL, argname);
       int x0 = LOGICAL(x)[0];
       PROTECT(x = Rf_allocVector(LGLSXP, 1));
+      nprotect++;
       LOGICAL(x)[0] = x0;
-      UNPROTECT(2);
+      UNPROTECT(nprotect);
       return x;
    }
    else { // if (nx == 1)
-      UNPROTECT(1);
+      UNPROTECT(nprotect);
       return x;
    }
 }
@@ -790,6 +1022,36 @@ double stri__prepare_arg_double_1_notNA(SEXP x, const char* argname)
 }
 
 
+/**
+ * This is a helper function to avoid UNPROTECTED var names warning
+ * when playing with CHARSXP directly
+ *
+ * @param x an R STRING object (from STRING_ELT(charactervector, index))
+ * @return an Ralloc'd character string
+ *
+ * @version 1.1.6 (Marek Gagolewski, 2017-11-10)
+ */
+const char* stri__copy_string_Ralloc(SEXP x, const char* argname)
+{
+   PROTECT(x);
+   if (x == NA_STRING) {
+      UNPROTECT(1);
+      Rf_error(MSG__ARG_EXPECTED_NOT_NA, argname); // allowed here
+   }
+
+   const char* ret_tmp = (const char*)CHAR(x); // ret may be gc'ed
+   size_t ret_n = strlen(ret_tmp);
+   /* R_alloc ==  Here R will reclaim the memory at the end of the call to .Call */
+   char* ret = R_alloc(ret_n+1, (int)sizeof(char));
+   if (!ret) {
+      UNPROTECT(1);
+      Rf_error(MSG__MEM_ALLOC_ERROR);
+   }
+   memcpy(ret, ret_tmp, ret_n+1);
+   UNPROTECT(1);
+   return ret;
+}
+
 /** Prepare double argument - one value, not NA [no re-encoding done!!!]
  *
  * If there are 0 elements -> error
